import unittest
import re

import raimad as rai

from .utils import GeomsEqual

class TestProxy(GeomsEqual, unittest.TestCase):

    def check_tower(self, proxy, expected):
        try:
            tower_actual = tuple(proxy.descend_p())
            self.assertEqual(len(tower_actual), len(expected))
            for \
                    this_proxy, (expected_autogenned, expected_deepcopied) \
                    in zip(tower_actual, expected, strict=True):

                #print(expected_autogenned, this_proxy.autogenned, expected_deepcopied, this_proxy.deepcopied)
                self.assertEqual(expected_autogenned, this_proxy.autogenned)
                self.assertEqual(expected_deepcopied, this_proxy.deepcopied)

        except AssertionError as err:
            print("EXPECTED: ")
            print(expected)
            print("ACTUAL: ")
            print(proxy)
            raise err

    def test_proxy_origin(self):
        """
        Test that the "origin" of a proxy is tracked correctly.
        By "origin" I mean whether the proxy was created explicitly
        or automatically (Proxy.autogenned),
        and whether it was a result of a deepcopy operation
        (Proxy.deepcopied)
        """

        # Just a regular proxy of a compo,
        # should be labeled not autogenned
        # (because we explicitly create it)
        compo = rai.Circle(10)
        p1 = compo.proxy()

        self.check_tower(p1, (
            (False, False),
            ))

        # The `.subcompos` of a component should return
        # directly the proxies that were assigned into it,
        # which were created explicitly in the _make function,
        # so should be marked not autogenned

        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.c = rai.Circle(10).proxy()

        mycompo = MyCompo()

        self.check_tower(mycompo.subcompos.c, (
            (False, False),
            ))

        # `.subcompos` of a *proxy*, on the other hand,
        # should autogenerate proxies on top of the manual proxies.
        # It should be created via Proxy.deep_copy_reassign, since we can do
        # `.subcompos` on `.subcompos`,
        # but it should not be marked a `deepcopied`,
        # since it's the top of the tower

        mycompo_p = mycompo.proxy()

        self.check_tower(mycompo_p.subcompos.c, (
            (True, False),
            (False, False),
            ))

        # Okay, but now if the use `Proxy.subcompos.subcompos`,
        # the result should be marked both autogenned and deepcopied,
        # because it was created as the result of a deepcopy
        class MyUpperCompo(rai.Compo):
            def _make(self):
                self.subcompos.m = MyCompo().proxy()

        my_upper_compo = MyUpperCompo()

        # TODO is this good?
        self.check_tower(my_upper_compo.subcompos.m.subcompos.c, (
            (True, False),
            (False, False),
            ))

        # TODO is this good?
        my_upper_compo_p = my_upper_compo.proxy()
        self.check_tower(my_upper_compo_p.subcompos.m.subcompos.c, (
            (True, False),
            (True, True),
            (False, False),
            ))

    def test_proxy_str(self):

        compo = rai.Snowman()
        p = compo.proxy().proxy().deep_copy()
        p2 = p.subcompos.base.proxy()

        self.assertTrue(
            re.match(
                r'^<Manual Proxy at .* with <.*> of\s*'
                r'Automatic Proxy at .* with <.*> of\s*'
                r'Automatic deepcopied Proxy at .* with <.*> of\s*'
                r'Manual Proxy at .* with <.*> of\s*'
                r'Circle at .*>'
                ,
                str(p2)
                )
            )
        self.assertEqual(str(p2), repr(p2))

    def test_proxy_scale(self):
        rect = rai.RectLW(10, 10).proxy().bbox.mid.to((0, 0))

        horiz = rect.proxy().scale(2, 1)
        vert = rect.proxy().scale(1, 2)
        both = rect.proxy().scale(2, 2)
        both_alt = rect.proxy().scale(2)

        self.assertGeomsEqual(
            rect.steamroll(),
            {'root': [[(-5, -5), (5, -5), (5, 5), (-5, 5)]]}
            )

        self.assertGeomsEqual(
            horiz.steamroll(),
            {'root': [[(-10, -5), (10, -5), (10, 5), (-10, 5)]]}
            )

        self.assertGeomsEqual(
            vert.steamroll(),
            {'root': [[(-5, -10), (5, -10), (5, 10), (-5, 10)]]}
            )

        self.assertGeomsEqual(
            both.steamroll(),
            {'root': [[(-10, -10), (10, -10), (10, 10), (-10, 10)]]}
            )

        self.assertGeomsEqual(
            both_alt.steamroll(),
            {'root': [[(-10, -10), (10, -10), (10, 10), (-10, 10)]]}
            )


if __name__ == '__main__':
    unittest.main()




import unittest
from contextlib import redirect_stderr
import shlex
import os
import subprocess
from io import StringIO
import tempfile
import random
from pathlib import Path

import raimad as rai

class TestCLI(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.snowman_cif = rai.export_cif(rai.Snowman())

    def test_cli_export_cif_noargs(self):
        pwd = os.getcwd()
        with tempfile.TemporaryDirectory() as folder:
            os.chdir(folder)
            subprocess.run(shlex.split(
                'python -m raimad export raimad:Snowman'
                ), check=True)

            cif_string = Path('Snowman.cif').read_text()
        os.chdir(pwd)

        self.assertEqual(self.snowman_cif, cif_string)

    def test_cli_export_cif_file(self):
        pwd = os.getcwd()
        with tempfile.TemporaryDirectory() as folder:
            os.chdir(folder)
            subprocess.run(shlex.split(
                'python -m raimad export raimad:Snowman -o compo.cif'
                ), check=True)

            cif_string = Path('compo.cif').read_text()
        os.chdir(pwd)

        self.assertEqual(self.snowman_cif, cif_string)

    def test_cli_fortune(self):
        result = subprocess.run(shlex.split(
            'python -m raimad fortune'
            ),
            check=True,
            capture_output=True,
            )

        fortune = result.stdout.decode('utf-8')
        self.assertIn(
            fortune.strip(),
            map(lambda s: s.strip(), rai.fortunes_all)
            )

    def test_cli_fortune_category(self):
        result = subprocess.run(shlex.split(
            'python -m raimad fortune resilience'
            ),
            check=True,
            capture_output=True,
            )

        fortune = result.stdout.decode('utf-8')
        self.assertIn(
            fortune.strip(),
            map(lambda s: s.strip(), rai.fortunes_resilience)
            )
        self.assertNotIn(
            fortune.strip(),
            map(lambda s: s.strip(), rai.fortunes_politics)
            )

# TODO other formats? svg?

if __name__ == '__main__':
    unittest.main()




"""
Test layer handling.

More higher-level version of test_lmap.py
"""

import unittest

import raimad as rai

class TestLayers(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def test_layers_root(self):
        """
        Test that a rectlw with no map ends up on root
        """
        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    rai.RectLW(10, 10).proxy()
                    )

        self.assertEqual(
            set(MyCompo().steamroll().keys()),
            {'root'}
            )

    def test_layers_root_none(self):
        """
        Test that a rectlw with an explicit None lmap ends up on root
        """
        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    rai.RectLW(10, 10)
                    .proxy()
                    .map(None)
                    )

        self.assertEqual(
            set(MyCompo().steamroll().keys()),
            {'root'}
            )

    def test_layers_root_stack(self):
        """
        Test that a "stack" of subcompos with no maps end up on root
        """
        class First(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    rai.RectLW(10, 10)
                    .proxy()
                    )

        class Second(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    First()
                    .proxy()
                    )

        class Third(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    Second()
                    .proxy()
                    )

        self.assertEqual(
            set(Third().steamroll().keys()),
            {'root'}
            )

    def test_layers_root_stack_complex(self):
        """
        Test a "stack" of subcompos, each with a one-to-one lmap
        """
        class First(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    rai.RectLW(10, 10)
                    .proxy()
                    .map({'root': 'foot'})
                    )

        class Second(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    First()
                    .proxy()
                    .map({'foot': 'toot'})
                    )

        class Third(rai.Compo):
            def _make(self):
                self.subcompos.append(
                    Second()
                    .proxy()
                    .map({'toot': 'boot'})
                    )

        self.assertEqual(
            set(Third().steamroll().keys()),
            {'boot'}
            )


if __name__ == '__main__':
    unittest.main()




"""Test that the .subcompos attribute works properly."""

import unittest

import raimad as rai

class TestSubcompoIntrospection(unittest.TestCase):

    def test_subcompo_introspection_basic(self):
        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.a = rai.RectLW(10, 10).proxy()

        compo = MyCompo()
        self.assertTrue(
            rai.eq(
                compo.bbox.top_left,
                compo.subcompos.a.bbox.top_left
                )
            )

    def test_subcompo_introspection_proxy(self):
        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.a = (
                    rai.RectLW(10, 10)
                    .proxy()
                    .move(10, 0)
                    )

        compo = MyCompo()
        self.assertTrue(
            rai.eq(
                compo.bbox.top_left,
                compo.subcompos.a.bbox.top_left
                )
            )

    def test_subcompo_introspection_proxy_proxy(self):
        class MyCompo(rai.Compo):
            def _make(self):
                self.subcompos.a = (
                    rai.RectLW(10, 10)
                    .proxy()
                    .move(10, 0)
                    )

        compo = MyCompo()
        proxy = compo.proxy().move(0, 10)

        self.assertTrue(
            rai.eq(
                compo.bbox.top_left,
                compo.subcompos.a.bbox.top_left
                )
            )

        self.assertTrue(
            rai.eq(
                proxy.bbox.top_left,
                proxy.subcompos.a.bbox.top_left
                )
            )

        self.assertFalse(
            rai.eq(
                compo.bbox.top_left,
                proxy.subcompos.a.bbox.top_left
                )
            )

        self.assertFalse(
            rai.eq(
                proxy.bbox.top_left,
                compo.subcompos.a.bbox.top_left
                )
            )




if __name__ == '__main__':
    unittest.main()

